# -*- coding: UTF-8 -*-
# FlameBot, Python IRC bot modular and extensible
#           FreeBSD data access
#
# Copyright (c) 2005 Diego Pettenò
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following
#   disclaimer in the documentation and/or other materials provided
#   with the distribution.
# - The name of the author may not be used to endorse or promote
#   products derived from this software without specific prior
#   written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from xml.dom import minidom
from string import Template

import re

# Local imports
from flamebot_utils import irc_answer, FlameBotOpener

pr_answer = Template("FBSD Problem Report ${category}/${number}; \"${synopsis}\"; ${severity}; ${state}; ${originator} -> ${responsible}; ${url}")

pr_regexps = {
	'category': re.compile(">Category: +(.+)"),
	'number': re.compile(">Number: +(.+)"),
	'synopsis': re.compile(">Synopsis: +(.+)"),
	'severity': re.compile(">Severity: +(.+)"),
	'state': re.compile(">State: +(.+)"),
	'originator': re.compile(">Originator: +(.+)"),
	'responsible': re.compile(">Responsible: +(.+)"),
	}

sa_answer = Template("FBSD Security Advisory FreeBSD-SA-${code}.${module} ${announced} [${category}]: ${topic} - ${cvename}; ${url}")

sa_regexps = {
	'module': re.compile("Module: +(.+)"),
	'announced': re.compile("Announced: +(.+)"),
	'category': re.compile("Category: +(.+)"),
	'topic': re.compile("Topic: +(.+)"),
	'cvename': re.compile("CVE Name: +(.+)"),
	}

def cmd_freebsdpr(connection, nick, channel, arg):
	url = "http://www.freebsd.org/cgi/query-pr.cgi?pr=" + arg + "&f=raw"
	print "Going to request " + url
	
	rawdata = FlameBotOpener().open(url).read()
	
	data = { }
	
	for value in pr_regexps.keys():
		match = pr_regexps[value].search(rawdata)
		if match != None:
			data[value] = match.group(1)
	
	data['url'] = "http://www.freebsd.org/cgi/query-pr.cgi?pr=" + arg
	
	irc_answer(connection, nick, channel, pr_answer.substitute(data))

def cmd_freebsdsa(connection, nick, channel, arg):
	url = "ftp://ftp.freebsd.org/pub/FreeBSD/CERT/advisories/FreeBSD-SA-" + arg + ".asc"
	print "Going to request " + url
	
	rawdata = FlameBotOpener().open(url).read()
	
	data = { 'code': arg }
	
	for value in sa_regexps.keys():
		match = sa_regexps[value].search(rawdata)
		if match != None:
			data[value] = match.group(1)
	
	data['url'] = "ftp://ftp.freebsd.org/pub/FreeBSD/CERT/advisories/FreeBSD-SA-" + arg + ".asc"
	
	irc_answer(connection, nick, channel, sa_answer.substitute(data))

def register_flamebot(bot):
	bot.addcommand("freebsdpr", cmd_freebsdpr)
	bot.addcommand("freebsdsa", cmd_freebsdsa)

def unregister(bot):
	bot.removecommand("freebsdpr")
	bot.removecommand("freebsdsa")
